/**
 *手机数据库
 * table_login 登录缓存表
 * table_postaddress 物业数据访问地址表
 * table_caseparams 报事参数缓存表
 * table_familybase 用户基本信息表
 * table_ownerhouse 用户房屋关系绑定表
 * table_shoppingcar 购物车表
 * table_attention 加关注表
 * table_goodscategory 商品分类表
 * table_searchHistory 搜索历史表
 * table_orderLog 订单记录表
 * table_netGoodsTime 净菜主表
 * table_netGoods 净菜商品表
 * table_appMenu app菜单表
 * table_distinct 折扣信息缓存表
 * table_waimai 外卖表
 * table_doors 门禁
 * table_areaPhone
 */
var FamilyDB = (function() {
	var connection = {
		cmd : null,
		open : function() {
			connection.cmd = connection.cmd ? connection.cmd : new DB();
			var flag = connection.cmd.isOpen();
			if (!flag) {
				connection.cmd.open("family.db", "sys:db/");
			}
		},
		close : function() {
			connection.cmd = connection.cmd ? connection.cmd : new DB();
			var flag = connection.cmd.isOpen();
			if (flag) {
				connection.cmd.close();
			}
		}
	};

	/**
	 *创建登录缓存表
	 */
	var _createLoginTable = function() {
		if (!connection.cmd.isTableExist("table_login")) {
			var array = [];
			array.push("loginparam varchar");
			connection.cmd.createTable("table_login", array);
		}
	};

	/**
	 *设置登录信息
	 */
	var _setLoginInfo = function(params) {
		connection.open();
		_createLoginTable();
		var sqls = [];
		sqls.push("delete from table_login");
		sqls.push("insert into table_login (loginparam) values ('" + EncryptionUtil.base64Encode(params) + "')");
		var result = connection.cmd.executes(sqls);
		connection.close();
		return result;
	};

	/**
	 *获取登录信息
	 */
	var _getLoginInfo = function() {
		connection.open();
		var sql = "select loginparam from table_login";
		var data = connection.cmd.query(sql);
		connection.close();
		return data.length > 0 ? eval("(" + EncryptionUtil.base64Decode(data[0].loginparam) + ")") : null;
	};

	/**
	 *获取客户基本信息
	 */
	var _getClientBase = function() {
		var loginInfo = _getLoginInfo();
		return loginInfo ? loginInfo.body.loginInfo : null;
	};

	/**
	 *创建数据访问地址表
	 */
	var _createPostAddressTable = function() {
		if (!connection.cmd.isTableExist("table_postaddress")) {
			var array = [];
			array.push("url varchar");
			connection.cmd.createTable("table_postaddress", array);
		}
	};

	/**
	 *设置物业数据访问地址
	 * @param {Object} address
	 */
	var _setPostAddress = function(address) {
		connection.open();
		_createPostAddressTable();
		var sqls = [];
		sqls.push("delete from table_postaddress");
		sqls.push("insert into table_postaddress (url) values ('" + EncryptionUtil.base64Encode(address) + "')");
		var result = connection.cmd.executes(sqls);
		connection.close();
		return result;
	};

	/**
	 * 获取物业数据访问地址
	 */
	var _getPostAddress = function() {
		connection.open();
		_createPostAddressTable();
		var sql = "select url from table_postaddress";
		var data = connection.cmd.query(sql);
		connection.close();
		return data.length > 0 ? EncryptionUtil.base64Decode(data[0].url) : null;
	};

	/**
	 *创建报事参数缓存表
	 */
	var _createCaseParamsTable = function() {
		if (!connection.cmd.isTableExist("table_caseparams")) {
			var array = [];
			array.push("params varchar");
			array.push("areaId varchar");
			array.push("insertTime varchar");
			connection.cmd.createTable("table_caseparams", array);
		}
		var flag = connection.cmd.addCol("table_caseparams", "areaId", "varchar");
		if (flag) {
			connection.close();
			connection.open();
		}
	};

	/**
	 *缓存报事参数 有效期1天
	 */
	var _setCaseParams = function(params, areaId) {
		connection.open();
		_createCaseParamsTable();
		var date = new Date();
		var time = date.getFullYear() + "-" + (date.getMonth() + 1) + "-" + date.getDate();
		var sqls = ["insert into table_caseparams (params,insertTime,areaId) values ('" + EncryptionUtil.base64Encode(params) + "','" + time + "','" + (areaId || "") + "')"];
		var result = connection.cmd.executes(sqls);
		connection.close();
		return result;
	};

	/**
	 * 获取报事参数
	 */
	var _getCaseParams = function(areaId) {
		connection.open();
		_createCaseParamsTable();
		var date = new Date();
		var time = date.getFullYear() + "-" + (date.getMonth() + 1) + "-" + date.getDate();
		var sqls = ["delete from table_caseparams where areaId='" + (areaId || "") + "' and insertTime<>'" + time + "'"];
		connection.cmd.executes(sqls);
		var sql = "select params from table_caseparams where areaId='" + (areaId || "") + "' and insertTime='" + time + "'";
		var data = connection.cmd.query(sql);
		connection.close();
		return data.length == 0 ? null : eval("(" + EncryptionUtil.base64Decode(data[0].params) + ")");
	};

	/**
	 *创建主信息缓存表
	 */
	var _createFamilyBaseTable = function() {
		if (!connection.cmd.isTableExist("table_familybase")) {
			var array = [];
			//当前使用模式 1已注册用户  0游客
			array.push("pattern integer");

			//当前所在电商小区id
			array.push("areaId varchar");

			//当前物业公司key
			array.push("propertyKey varhcar");

			//当前物业小区id
			array.push("esareaId varchar");

			//当前小区
			array.push("areaName varchar");

			//是否开通电商服务
			array.push("openOnline interger");
			connection.cmd.createTable("table_familybase", array);
		}
	};

	/**
	 * 设置主信息{pattern:"",areaId:"",propertyKey:"",esareaId:""}
	 * @param {Object} option
	 */
	var _setFamilyBase = function(options) {
		connection.open();
		_createFamilyBaseTable();
		var sqls = [];
		sqls.push("delete from table_familybase");
		var sql = "";
		sql = "insert into table_familybase (pattern,areaId,propertyKey,esareaId,areaName,openOnline) values (";
		sql += options.pattern + ",";
		sql += "'" + options.areaId + "',";
		sql += "'" + options.propertyKey + "',";
		sql += "'" + options.esareaId + "',";
		sql += "'" + options.areaName + "',";
		sql += options.openOnline;
		sql += ")";
		sqls.push(sql);
		var result = connection.cmd.executes(sqls);
		connection.close();
		return result;
	};

	/**
	 *获取主信息
	 */
	var _getFamilyBase = function() {
		connection.open();
		_createCaseParamsTable();
		var sql = "select * from table_familybase";
		var data = connection.cmd.query(sql);
		connection.close();
		return data.length == 0 ? {} : data[0];
	};

	//检测游客
	var _checkPattern = function() {
		var _base = _getFamilyBase();
		return _base ? (_base.pattern == 0) : false;
	};

	/**
	 *创建业主房屋表
	 */
	var _createHouseTable = function() {
		if (!connection.cmd.isTableExist("table_ownerhouse")) {
			var array = [];
			array.push("id varchar");
			array.push("areaId varchar");
			array.push("objectId varchar");
			array.push("objectName varchar");
			array.push("ownerId varchar");
			array.push("ownerName varchar");
			array.push("clientId varchar");
			array.push("esareaId varchar");
			connection.cmd.createTable("table_ownerhouse", array);
		}
	};

	/**
	 *设置业主绑定房屋
	 */
	var _setOwnerHouse = function(clientId, datas) {
		connection.open();
		_createHouseTable();
		var sqls = [];
		sqls.push("delete from table_ownerhouse where clientId='" + clientId + "'");
		for (var i = 0; i < datas.length; i++) {
			var data = datas[i];
			sqls.push("insert into table_ownerhouse(id,areaId,objectId,objectName,ownerId,ownerName,clientId,esareaId) values ('" + data.id + "','" + data.areaId + "','" + data.objectId + "','" + data.objectName + "','" + data.ownerId + "','" + data.ownerName + "','" + data.clientId + "','" + data.esareaId + "')");
		}
		var result = connection.cmd.executes(sqls);
		connection.close();
		return result;
	};
	/**
	 * 获取业主绑定的房屋
	 * @param {Object} clientId 当前用户id
	 */
	var _getOwnerHouse = function(clientId) {
		connection.open();
		_createHouseTable();
		var sql = "select id,areaId,objectId,objectName,ownerId,ownerName,esareaId from table_ownerhouse where clientId='" + clientId + "'";
		var data = connection.cmd.query(sql);
		connection.close();
		return data.length == 0 ? null : data;
	};

	/**
	 * 获取业主绑定的房屋
	 * @param {Object} clientId 当前用户id
	 * @param {Object} areaId 小区id
	 */
	var _getOwnerHouseByArea = function(clientId, areaId) {
		connection.open();
		_createHouseTable();
		var sql = "select id,areaId,objectId,objectName,ownerId,ownerName,esareaId from table_ownerhouse where clientId='" + clientId + "' and areaId='" + areaId + "'";
		var data = connection.cmd.query(sql);
		connection.close();
		return data.length == 0 ? null : data;
	};

	/**
	 *检测是否在登录小区绑定了房屋
	 * @param {withOutOwner} true:内部员工等非业主信息不计算在内,false内部员工等非业主信息计算在内
	 */
	var _checkClientObjectDatas = function(withOutOwner) {
		try {
			var clientBase = FamilyDB.GetClientBase();
			var familyBase = FamilyDB.GetFamilyBase();
			var clientId = ( clientBase ? clientBase.id : FamilyCache.EmptyId);
			var areaId = ( familyBase ? familyBase.areaId : FamilyCache.EmptyId);
			connection.open();
			_createHouseTable();
			var sql = "select id,areaId,objectId,objectName,ownerId,ownerName,esareaId from table_ownerhouse where clientId='" + clientId + "' and areaId='" + areaId + "'" + ( withOutOwner ? " and objectName<>'内部员工'" : "");
			var data = connection.cmd.query(sql);
			connection.close();
			return data.length == 0 ? false : true;
		} catch(e) {
			return false;
		}
		return true;
	};

	/**
	 *是否为易软超级用户
	 */
	var _isEasySoft = function() {
		var clientBase = FamilyDB.GetClientBase();
		var familyBase = FamilyDB.GetFamilyBase();
		var clientId = ( clientBase ? clientBase.id : FamilyCache.EmptyId);
		var areaId = ( familyBase ? familyBase.areaId : FamilyCache.EmptyId);
		connection.open();
		_createHouseTable();
		var sql = "select * from table_ownerhouse where clientId='" + clientId + "' and areaId='14262790-45fc-42cd-a9b0-e8347ebb0a72'";
		var data = connection.cmd.query(sql);
		connection.close();
		return data.length == 0 ? false : true;
	};
	/**
	 *获取单个房屋信息
	 * @param {Object} clientId 当前用户id
	 * @param {Object} areaId 小区id
	 */
	var _getOneHouseData = function(clientId, areaId) {
		var datas = _getOwnerHouseByArea(clientId, areaId);
		return datas ? datas[0] : null;
	};

	/**
	 *创建购物车表
	 */
	var _createShoppingcarTable = function() {
		if (!connection.cmd.isTableExist("table_shoppingcar")) {
			var array = [];

			array.push("id varchar");

			//商品id
			array.push("goodsId varchar");

			//商品名称
			array.push("goodsName varchar");

			//售价
			array.push("goodsPrice decimal(18,2)");

			//购买数量
			array.push("buyNumber integer");

			//配送方式
			array.push("deliveryDesc varchar");

			//配送方式
			array.push("deliveryMode integer");

			//仓库id
			array.push("storageId varchar");

			//总价
			array.push("totalPrice decimal(18,2)");

			connection.cmd.createTable("table_shoppingcar", array);
		}
	};

	/**
	 *加入购物车
	 * @param {Object} goods json格式
	 */
	var _setShoppingcar = function(goods) {
		connection.open();
		_createShoppingcarTable();
		var sqls = [];
		var sql = "";
		if (!_selectShoppingcar(goods.goodsId)) {
			sql += " insert into table_shoppingcar (id,goodsId,goodsName,goodsPrice,buyNumber,deliveryDesc,deliveryMode,storageId,totalPrice)";
			sql += " values (";
			sql += "'" + Family.NewId() + "',";
			sql += "'" + goods.goodsId + "',";
			sql += "'" + goods.goodsName + "',";
			sql += "'" + goods.goodsPrice + "',";
			sql += "'" + goods.buyNumber + "',";
			sql += "'" + goods.deliveryDesc + "',";
			sql += "'" + goods.deliveryMode + "',";
			sql += "'" + goods.storageId + "',";
			sql += "'" + goods.totalPrice + "'";
			sql += " ) ";
			sqls.push(sql);
		} else {
			sql = " update table_shoppingcar set buyNumber=buyNumber+1,totalPrice=goodsPrice*(buyNumber+1) where goodsId='" + goods.goodsId + "'";
			sqls.push(sql);
		}
		var result = connection.cmd.executes(sqls);
		connection.close();
		return result;
	};

	/**
	 *
	 */
	var _selectShoppingcar = function(goodsId) {
		var sql = "select * from table_shoppingcar where goodsId='" + goodsId + "'";
		var data = connection.cmd.query(sql);
		return data.length == 0 ? null : data;
	};

	/**
	 *获取购物车
	 */
	var _getShoppingcar = function() {
		connection.open();
		_createShoppingcarTable();
		var sql = "select goodsId,goodsName,cast(goodsPrice as decimal(18,2)) goodsPrice,buyNumber,deliveryDesc,deliveryMode,storageId,(goodsPrice*buyNumber) as totalPrice from table_shoppingcar";
		var data = connection.cmd.query(sql);
		connection.close();
		return data.length == 0 ? null : data;
	};

	/**
	 *删除购物车
	 * @param {Object} goodsIdList
	 */
	var _deleteShoppingcar = function(goodsIdList) {
		connection.open();
		_createShoppingcarTable();
		var sqls = ["delete from table_shoppingcar where goodsId in (" + goodsIdList + ")"];
		var result = connection.cmd.executes(sqls);
		connection.close();
		return result;
	};

	/**
	 *添加关注
	 */
	var _createAttentionTable = function() {
		if (!connection.cmd.isTableExist("table_attention")) {
			var array = [];
			array.push("id varchar");

			//1商品
			array.push("type integer");

			//关注信息 json字符串
			array.push("attentionInfo varchar");

			connection.cmd.createTable("table_attention", array);
		}
	};

	/**
	 *加关注
	 * @param {Object} options
	 */
	var _setAttention = function(options) {
		connection.open();
		_createAttentionTable();
		var sqls = [];
		var sql = "insert into table_attention (id,type,attentionInfo) values (";
		sql += "'" + options.id + "',";
		sql += options.type + ",";
		sql += "'" + EncryptionUtil.base64Encode(options.attentionInfo) + "'";
		sql += ")";
		sqls.push(sql);
		var result = connection.cmd.executes(sqls);
		connection.close();
		return result;
	};

	/**
	 *获取关注
	 */
	var _getAttention = function(id) {
		connection.open();
		_createAttentionTable();
		var sql = "select id,type,attentionInfo from table_attention where id='" + id + "'";
		var data = connection.cmd.query(sql);
		for (var i = 0; i < data.length; i++) {
			data[i].attentionInfo = EncryptionUtil.base64Decode(data[i].attentionInfo);
		}
		connection.close();
		return data.length == 0 ? null : data;
	};

	/**
	 *取消关注
	 */
	var _deleteAttention = function(id) {
		connection.open();
		_createAttentionTable();
		var sqls = [];
		sqls.push("delete from table_attention where id='" + id + "'");
		var result = connection.cmd.executes(sqls);
		connection.close();
		return result;
	};

	/**
	 *商品分类表
	 */
	var _createCategoryTable = function() {
		if (!connection.cmd.isTableExist("table_goodscategory")) {
			var array = [];
			array.push("id varchar");
			array.push("pid varchar");
			array.push("name varchar");
			array.push("level integer");
			array.push("insertDate varchar");
			connection.cmd.createTable("table_goodscategory", array);
		}
	};

	/**
	 *设置商品分类
	 * @param {Object} categoryDatas
	 */
	var _setCategory = function(categoryDatas) {
		connection.open();
		_createCategoryTable();
		var date = new Date();
		var time = date.getFullYear() + "-" + (date.getMonth() + 1) + "-" + date.getDate();
		var sqls = [];
		sqls.push("delete from table_goodscategory");
		for (var i = 0; i < categoryDatas.length; i++) {
			var categoryInfo = categoryDatas[i];
			var sql = "insert into table_goodscategory (id,pid,name,level,insertDate) values (";
			sql += "'" + categoryInfo.id + "',";
			sql += "'" + categoryInfo.pid + "',";
			sql += "'" + categoryInfo.name + "',";
			sql += "'" + categoryInfo.level + "',";
			sql += "'" + time + "'";
			sql += ")";
			sqls.push(sql);
		}
		var result = connection.cmd.transaction(sqls);
		connection.close();
		return result;
	};

	/**
	 *获取商品分类
	 */
	var _getCategory = function() {
		connection.open();
		_createCategoryTable();
		var date = new Date();
		var time = date.getFullYear() + "-" + (date.getMonth() + 1) + "-" + date.getDate();
		var sqls = ["delete from table_goodscategory where insertDate<>'" + time + "'"];
		connection.cmd.executes(sqls);
		var sql = "select id,pid,name,level from table_goodscategory where insertDate='" + time + "'";
		var data = connection.cmd.query(sql);
		connection.close();
		return data.length == 0 ? null : data;
	};

	/**
	 * 获取商品分类
	 * @param {Object} pid 父节点id
	 */
	var _getCategoryByParent = function(pid) {
		connection.open();
		_createCategoryTable();
		var sql = "select id,pid,name,level from table_goodscategory where pid='" + pid + "'";
		var data = connection.cmd.query(sql);
		connection.close();
		return data.length == 0 ? null : data;
	};

	/**
	 *创建搜索历史表
	 */
	var _createSearchHistoryTable = function() {
		if (!connection.cmd.isTableExist("table_searchhistory")) {
			var array = [];
			array.push("id varchar");

			array.push("name varchar");

			//1商品搜索历史
			array.push("type integer");

			//预留字段 已做加密处理
			array.push("otherInfo varchar");
			connection.cmd.createTable("table_searchHistory", array);
		}
	};

	/**
	 *设置历史搜索
	 */
	var _setSearchHistory = function(options) {
		connection.open();
		_createSearchHistoryTable();
		var sqls = [];
		sqls.push("delete from table_searchhistory where name='" + options.name + "'");

		var sql = "insert into table_searchhistory (id,name,type,otherInfo) values (";
		sql += "'" + (options.id || "") + "',";
		sql += "'" + (options.name || "") + "',";
		sql += (options.type || 1) + ",";
		sql += "'" + EncryptionUtil.base64Encode((options.otherInfo || "")) + "'";
		sql += ")";

		sqls.push(sql);

		var result = connection.cmd.executes(sqls);
		connection.close();
		return result;
	};

	/**
	 *获取历史搜索
	 */
	var _getSearchHistory = function(type) {
		connection.open();
		_createSearchHistoryTable();
		var sql = "select * from table_searchhistory where type=" + type;
		var data = connection.cmd.query(sql);
		connection.close();
		return data.length == 0 ? null : data;
	};

	/**
	 *删除历史搜索
	 */
	var _deleteSearchHistory = function() {
		connection.open();
		_createSearchHistoryTable();
		var sqls = [];
		var sql = "delete from table_searchhistory " + (arguments[0] ? "where type=" + arguments[0] : "");
		sqls.push(sql);
		var result = connection.cmd.executes(sqls);
		connection.close();
		return result;
	};

	/**
	 *创建订单记录表
	 */
	var _createOrderLogTable = function() {
		if (!connection.cmd.isTableExist("table_orderlog")) {
			var array = [];
			array.push("params varchar");
			connection.cmd.createTable("table_orderlog", array);
		}
	};

	/**
	 *订单缓存记录
	 * @param {Object} params
	 */
	var _setOrderLog = function(params) {
		connection.open();
		_createOrderLogTable();
		var sqls = [];
		sqls.push("delete from table_orderlog");

		var sql = "insert into table_orderlog (params) values (";
		sql += "'" + EncryptionUtil.base64Encode((params || "")) + "'";
		sql += ")";
		sqls.push(sql);

		var result = connection.cmd.executes(sqls);
		connection.close();
		return result;
	};

	/**
	 *获取订单缓存
	 */
	var _getOrderLog = function() {
		connection.open();
		_createOrderLogTable();
		var sql = "select * from table_orderlog";
		var data = connection.cmd.query(sql);
		connection.close();
		return data.length == 0 ? null : eval("(" + EncryptionUtil.base64Decode(data[0].params) + ")");
	};

	/**
	 *创建净菜主表
	 */
	var _createTableNetGoodsTime = function() {
		if (!connection.cmd.isTableExist("table_netGoodsTime")) {
			var array = [];
			array.push("id varchar");
			array.push("timespan varchar");
			connection.cmd.createTable("table_netGoodsTime", array);
		}
	};

	/**
	 *设置净菜选购日期
	 * @param {Object} options
	 */
	var _setNetGoodsTime = function(options) {
		connection.open();
		_createTableNetGoodsTime();
		var sqls = [];
		var sql = "insert into table_netGoodsTime (id,timespan) values (";
		sql += "'" + options.id + "',";
		sql += "'" + options.timespan + "'";
		sql += ")";
		sqls.push(sql);

		var result = connection.cmd.executes(sqls);
		connection.close();
		return result;
	};
	/**
	 *获取净菜选购日期
	 * @param {Object} options
	 */
	var _getNetGoodsTime = function(options) {
		connection.open();
		_createTableNetGoodsTime();
		var sql = "select * from table_netGoodsTime";
		var data = connection.cmd.query(sql);
		connection.close();
		return data.length == 0 ? null : data;
	};

	/**
	 *检测是否已存在净菜选购日期
	 * @param {Object} time
	 */
	var _isExistsNetGoodsTime = function(timespan) {
		connection.open();
		_createTableNetGoodsTime();
		var sql = "select * from table_netGoodsTime where timespan='" + timespan + "'";
		var data = connection.cmd.query(sql);
		connection.close();
		return data.length == 0;
	};

	/**
	 * 删除净菜选购日期
	 */
	var _deleteNetGoodsTime = function() {
		connection.open();
		_createTableNetGoodsTime();
		var sqls = [];
		var sql = "delete from table_netGoodsTime";
		sqls.push(sql);

		var result = connection.cmd.executes(sqls);
		connection.close();
		return result;
	};

	var _createTableNetGoods = function() {
		if (!connection.cmd.isTableExist("table_netGoods")) {
			var array = [];
			array.push("id varchar");
			array.push("goodsId varchar");
			array.push("goodsName varchar");
			array.push("goodsPrice varchar");
			array.push("buyNumber varchar");
			array.push("timespan varchar");
			array.push("goodsImg varchar");
			connection.cmd.createTable("table_netGoods", array);
		}
	};

	/**
	 *设置净菜商品
	 * @param {Object} datas
	 */
	var _setNetGoods = function(datas) {
		connection.open();
		_createTableNetGoods();
		var sqls = [];
		for (var i = 0; i < datas.length; i++) {
			var options = datas[i];
			var sql = "insert into table_netGoods (id,goodsId,goodsName,goodsPrice,buyNumber,timespan,goodsImg) values (";
			sql += "'" + options.id + "',";
			sql += "'" + options.goodsId + "',";
			sql += "'" + options.goodsName + "',";
			sql += "'" + options.goodsPrice + "',";
			sql += "'" + options.buyNumber + "',";
			sql += "'" + options.timespan + "',";
			sql += "'" + options.goodsImg + "'";
			sql += ")";
			sqls.push(sql);
		}

		var result = connection.cmd.executes(sqls);
		connection.close();
		return result;
	};

	/**
	 *获取净菜商品
	 */
	var _getNetGoods = function() {
		var datas = _getNetGoodsTime();
		var goodsData = _selectNetGoods();
		var str = "[";
		if (datas && goodsData) {
			for (var i = 0; i < datas.length; i++) {
				var timeData = datas[i];
				str += "{";
				str += "\"time\":\"" + timeData.timespan + "\",";
				str += "\"goodsData\":[";
				for (var j = 0; j < goodsData.length; j++) {
					var goods = goodsData[j];
					if (goods.timespan == timeData.timespan) {
						str += "{";
						str += "\"id\":\"" + goods.id + "\",";
						str += "\"goodsId\":\"" + goods.goodsId + "\",";
						str += "\"goodsName\":\"" + goods.goodsName + "\",";
						str += "\"goodsPrice\":\"" + goods.goodsPrice + "\",";
						str += "\"buyNumber\":\"" + goods.buyNumber + "\",";
						str += "\"timespan\":\"" + goods.timespan + "\",";
						str += "\"goodsImg\":\"" + goods.goodsImg + "\"";
						str += "},";
					}
				}
				str += "]";
				str += "},";
			}
		}
		str += "]";
		return str.toJSON();
	};

	var _selectNetGoods = function() {
		connection.open();
		_createTableNetGoodsTime();
		var sql = "select * from table_netGoods";
		var data = connection.cmd.query(sql);
		connection.close();
		return data.length == 0 ? null : data;
	};

	/**
	 *更新净菜商品
	 */
	var _updateNetGoods = function(options) {
		connection.open();
		_createTableNetGoods();
		var sqls = [];
		var sql = "update table_netGoods set buyNumber='" + options.buyNumber + "' where id='" + options.id + "'";
		sqls.push(sql);
		var result = connection.cmd.executes(sqls);
		connection.close();
		return result;
	};

	/**
	 *删除净菜商品
	 * @param {Object} id
	 */
	var _deleteNetGoods = function(id) {
		connection.open();
		_createTableNetGoods();
		var sqls = [];
		var sql = "delete from table_netGoods " + ( id ? " where id='" + id + "'" : "");
		sqls.push(sql);
		var result = connection.cmd.executes(sqls);
		connection.close();
		return result;
	};

	/**
	 *检测当前日期的净菜是否已经存在
	 * @param {Object} timespan
	 * @param {Object} goodsId
	 */
	var _isExitsNetGoods = function(timespan, goodsId) {
		connection.open();
		_createTableNetGoodsTime();
		var sql = "select * from table_netGoods where goodsId='" + goodsId + "' and timespan='" + timespan + "'";
		var data = connection.cmd.query(sql);
		connection.close();
		return data.length == 0;
	};

	/**
	 * 创建app菜单表
	 */
	var _createAppMenuTable = function() {
		if (!connection.cmd.isTableExist("table_appMenu")) {
			var array = [];
			array.push("menus varchar");
			connection.cmd.createTable("table_appMenu", array);
		}
	};

	/**
	 *设置app菜单
	 * @param {ObjectString}
	 */
	var _setAppMenu = function(params) {
		try {
			connection.open();
			_createAppMenuTable();
			var sqls = [];
			sqls.push("delete from table_appMenu");
			sqls.push("insert into table_appMenu (menus) values ('" + EncryptionUtil.base64Encode(ClientUtil.jsonToString(params)) + "')");
			var result = connection.cmd.executes(sqls);
			connection.close();
			return result;
		} catch(e) {
		}

	};

	/**
	 *获取app菜单
	 */
	var _getAppMenu = function() {
		connection.open();
		_createAppMenuTable();
		var sql = "select * from table_appMenu where menus<>''";
		var data = connection.cmd.query(sql);
		connection.close();
		return data.length == 0 ? null : ClientUtil.stringToJson(EncryptionUtil.base64Decode(data[0].menus));
	};

	/**
	 *创建折扣缓存信息
	 */
	var _createDistinctTable = function() {
		if (!connection.cmd.isTableExist("table_distinct")) {
			var array = [];
			array.push("datas varchar");
			array.push("currentGoodsInfo varchar");
			connection.cmd.createTable("table_distinct", array);
		}
	};

	/**
	 *设置折扣信息
	 * @param {Object} params 折扣信息
	 * @param {Object} currentGoodsInfo 当前够买商品信息
	 */
	var _setDistinctLog = function(params, currentGoodsInfo) {
		try {
			connection.open();
			_createDistinctTable();
			var sqls = [];
			sqls.push("delete from table_distinct");
			sqls.push("insert into table_distinct (datas,currentGoodsInfo) values ('" + EncryptionUtil.base64Encode(ClientUtil.jsonToString(params)) + "','" + EncryptionUtil.base64Encode(ClientUtil.jsonToString(currentGoodsInfo)) + "')");
			var result = connection.cmd.executes(sqls);
			connection.close();
			return result;
		} catch(e) {
		}
	};

	/**
	 *获取商品折扣信息
	 */
	var _getDistinctLog = function() {
		connection.open();
		_createDistinctTable();
		var sql = "select * from table_distinct";
		var data = connection.cmd.query(sql);
		connection.close();
		return data.length == 0 ? null : data[0];
	};

	//创建外卖订单表
	var _createWaimaiTable = function() {
		if (!connection.cmd.isTableExist("table_Waimai")) {
			var array = [];
			array.push("goodsId varchar");
			array.push("goodsName varchar");
			array.push("goodsPrice varchar");
			array.push("buyNumber varchar");
			connection.cmd.createTable("table_Waimai", array);
		}
	};

	/**
	 * 设置外面订单
	 * @param {Object} options
	 */
	var _setWaimaiOrder = function(datas) {
		connection.open();
		_createWaimaiTable();
		var sqls = [];
		for (var i = 0; i < datas.length; i++) {
			var options = datas[i];
			var sql = "insert into table_Waimai (goodsId,goodsName,goodsPrice,buyNumber) values (";
			sql += "'" + options.goodsId + "',";
			sql += "'" + options.goodsName + "',";
			sql += "'" + options.goodsPrice + "',";
			sql += "'" + options.buyNumber + "'";
			sql += ")";
			sqls.push(sql);
		}

		var result = connection.cmd.executes(sqls);
		connection.close();
	};

	/**
	 *删除外卖订单
	 */
	var _deleteWaimaiOrder = function() {
		connection.open();
		_createWaimaiTable();
		var sqls = [];
		var sql = "delete from table_Waimai";
		sqls.push(sql);
		var result = connection.cmd.executes(sqls);
		connection.close();
	};

	/**
	 *获取外卖订单
	 */
	var _getWaimaiOrder = function() {
		connection.open();
		_createWaimaiTable();
		var sql = "select * from table_Waimai";
		var data = connection.cmd.query(sql);
		connection.close();
		return data.length == 0 ? null : data;
	};

	//创建门禁表
	var _createDoorsTable = function() {
		if (!connection.cmd.isTableExist("table_doors")) {
			var array = [];
			array.push("doorList varchar");
			connection.cmd.createTable("table_doors", array);
		}
	};

	/**
	 *设置门禁缓存信息
	 * @param {Object} datas
	 */
	var _setDoors = function(datas) {
		connection.open();
		_createDoorsTable();
		var sqls = [];
		sqls.push("delete from table_doors");
		sqls.push("insert into table_doors (doorList) values ('" + EncryptionUtil.base64Encode(ClientUtil.jsonToString(datas)) + "')");
		var result = connection.cmd.executes(sqls);
		connection.close();
		return result;
	};

	/**
	 * 获取门禁缓存信息
	 */
	var _getDoors = function() {
		connection.open();
		_createDoorsTable();
		var sql = "select doorList from table_doors";
		var data = connection.cmd.query(sql);
		connection.close();
		return data.length == 0 ? null : ClientUtil.stringToJson(EncryptionUtil.base64Decode(data[0].doorList));
	};

	//门禁设置表
	var _createDoorSettingsTable = function() {
		if (!connection.cmd.isTableExist("table_doorSettings")) {
			var array = [];
			array.push("params varchar");
			//1摇一摇设置
			array.push("type varchar");
			connection.cmd.createTable("table_doorSettings", array);
		}
	};

	/**
	 *门禁设置
	 * @param {Object} data
	 */
	var _setDoorSettings = function(data) {
		connection.open();
		_createDoorSettingsTable();
		var sqls = [];
		sqls.push("delete from table_doorSettings where type='" + data.type + "'");
		sqls.push("insert into table_doorSettings (params,type) values ('" + data.params + "','" + data.type + "')");
		var result = connection.cmd.executes(sqls);
		connection.close();
		return result;
	};

	/**
	 *获取门禁设置
	 * @param {Object} type
	 */
	var _getDoorSettings = function(type) {
		connection.open();
		_createDoorSettingsTable();
		var sql = "select * from table_doorSettings where type='" + type + "'";
		var data = connection.cmd.query(sql);
		connection.close();
		return data.length == 0 ? null : data[0];
	};

	/**
	 *删除门禁设置
	 * @param {Object} type
	 */
	var _deleteDoorSettings = function(type) {
		connection.open();
		_createDoorSettingsTable();
		var sqls = [];
		sqls.push("delete from table_doorSettings where type='" + type + "'");
		var result = connection.cmd.executes(sqls);
		connection.close();
		return result;
	};

	/**
	 *小区联系方式
	 */
	var _createAreaPhoneTable = function() {
		if (!connection.cmd.isTableExist("table_areaPhone")) {
			var array = [];
			array.push("areaId varchar");
			array.push("params varchar");
			connection.cmd.createTable("table_areaPhone", array);
		}
	};

	/**
	 *设置小区联系方式
	 */
	var _setAreaPhone = function(areaId, datas) {
		connection.open();
		_createAreaPhoneTable();
		var sqls = [];
		sqls.push("delete from table_areaPhone where areaId='" + areaId + "'");
		sqls.push("insert into table_areaPhone (areaId,params) values ('" + areaId + "','" + EncryptionUtil.base64Encode(ClientUtil.jsonToString(datas)) + "')");
		var result = connection.cmd.executes(sqls);
		connection.close();
		return result;
	};

	/**
	 *获取小区联系方式
	 * @param {Object} areaId
	 */
	var _getAreaPhone = function(areaId) {
		connection.open();
		_createAreaPhoneTable();
		var sql = "select * from table_areaPhone where areaId='" + areaId + "'";
		var data = connection.cmd.query(sql);
		connection.close();
		return data.length == 0 ? null : ClientUtil.stringToJson(EncryptionUtil.base64Decode(data[0].params));
	};

	/**
	 * 蓝牙门禁设备表
	 */
	var _createBluetoothTable = function() {
		if (!connection.cmd.isTableExist("table_bluetooth")) {
			var array = [];
			array.push("id varchar");
			array.push("name varchar");
			array.push("mac varchar");
			array.push("rssi varhcar");
			array.push("address nvarchar");
			connection.cmd.createTable("table_bluetooth", array);
		}
	};
	var _setBluetooth = function(data) {
		connection.open();
		_createBluetoothTable();
		var sqls = [];
		for (var i = 0; i < data.length; i++) {
			var dt = data[i];
			var sql = "insert into table_bluetooth(id,name,mac,rssi,address) values (";
			sql += "'" + (dt.id || "") + "',";
			sql += "'" + (dt.name || "") + "',";
			sql += "'" + (dt.mac || "") + "',";
			sql += "'" + (dt.rssi || "") + "',";
			sql += "'" + (dt.address || "") + "'";
			sql += ")";
			sqls.push(sql);
		}
		var result = connection.cmd.transaction(sqls);
		connection.close();
		return result;
	};
	var _getBluetoothMAX = function(params) {
		connection.open();
		_createBluetoothTable();
		var sql = "select *,max(rssi) mx from table_bluetooth where 1=1 and " + params;
		var data = connection.cmd.query(sql);
		connection.close();
		return data.length > 0 ? data : null;
	};
	var _delBluetooth = function() {
		connection.open();
		_createBluetoothTable();
		var sql = "delete from table_bluetooth";
		var data = connection.cmd.query(sql);
		connection.close();
		return data.length > 0 ? data : null;
	};
	return {
		SetLoginInfo : _setLoginInfo, //设置登录信息
		GetLoginInfo : _getLoginInfo, //获取登录信息
		GetClientBase : _getClientBase, //获取客户基本信息

		SetPostAddress : _setPostAddress, //设置物业数据访问地址
		GetPostAddress : _getPostAddress, //获取物业数据访问地址

		SetCaseParams : _setCaseParams, //缓存报事参数 有效期1天
		GetCaseParams : _getCaseParams, //获取报事参数

		SetFamilyBase : _setFamilyBase, //设置基本信息
		GetFamilyBase : _getFamilyBase, //获取基本信息

		SetOwnerHouse : _setOwnerHouse, //设置业主绑定的房屋
		GetOwnerHouse : _getOwnerHouse, //获取业主绑定的房屋
		GetOwnerHouseByArea : _getOwnerHouseByArea, //获取业主绑定的房屋
		GetOneHouseData : _getOneHouseData, //获取单个房屋数据
		CheckClientObjectDatas : _checkClientObjectDatas, //检测是否在登录小区绑定了房屋
		IsEasySoft : _isEasySoft, //是否为易软小区人员

		SetShoppingcar : _setShoppingcar, //加入购物车
		GetShoppingcar : _getShoppingcar, //获取购物车
		DeleteShoppingcar : _deleteShoppingcar, //删除购物车

		SetAttention : _setAttention, //加关注
		GetAttention : _getAttention, //获取关注
		DeleteAttention : _deleteAttention, //取消关注

		SetCategory : _setCategory, //设置商品分类
		GetCategory : _getCategory, //获取商品分类
		GetCategoryByParent : _getCategoryByParent, //获取商品分类

		SetSearchHistory : _setSearchHistory, //设置搜索历史
		GetSearchHistory : _getSearchHistory, //获取搜索历史
		DeleteSearchHistory : _deleteSearchHistory, //删除搜索历史

		SetOrderLog : _setOrderLog, //订单缓存记录
		GetOrderLog : _getOrderLog, //获取订单缓存记录

		SetNetGoodsTime : _setNetGoodsTime, //设置净菜选购日期
		GetNetGoodsTime : _getNetGoodsTime, //获取净菜选购日期
		IsExistsNetGoodsTime : _isExistsNetGoodsTime, //检测是否已存在净菜选购日期
		DeleteNetGoodsTime : _deleteNetGoodsTime, //删除净菜订购日期

		SetNetGoods : _setNetGoods, //设置净菜商品
		GetNetGoods : _getNetGoods, //获取净菜商品
		UpdateNetGoods : _updateNetGoods, //更新净菜商品
		DeleteNetGoods : _deleteNetGoods, //删除净菜商品
		SelectNetGoods : _selectNetGoods, //获取净菜商品
		IsExitsNetGoods : _isExitsNetGoods, //是否存在净菜商品

		SetAppMenu : _setAppMenu, //设置APP菜单
		GetAppMenu : _getAppMenu, //获取APP菜单

		SetDistinctLog : _setDistinctLog, //设置商品折扣信息
		GetDistinctLog : _getDistinctLog, //获取商品折扣信息

		SetWaimaiOrder : _setWaimaiOrder, //设置外卖订单
		DeleteWaimaiOrder : _deleteWaimaiOrder, //删除外卖订单
		GetWaimaiOrder : _getWaimaiOrder, //获取外卖订单

		CheckPattern : _checkPattern,

		SetDoors : _setDoors, //设置门禁缓存信息
		GetDoors : _getDoors, //获取门禁缓存信息

		SetDoorSettings : _setDoorSettings, //门禁设置
		GetDoorSettings : _getDoorSettings, //获取门禁设置
		DeleteDoorSettings : _deleteDoorSettings, //删除门禁设置

		SetAreaPhone : _setAreaPhone, //设置小区联系方式
		GetAreaPhone : _getAreaPhone, //获取小区联系方式

		SetBluetooth : _setBluetooth, //设置门禁设备缓存
		GetBluetoothMAX : _getBluetoothMAX, //获取门禁信号最强设备
		DelBluetooth : _delBluetooth //清楚门禁设备缓存
	};
})();
